%Simulation code for Table 2 of Kline and Santos (2011)

clear

randn('seed',1234)
rand('seed',1234)


R=10000; %number of simulations
R2=200; %bootstrap reps
K=3;  %number of misspecification points
S=2; %number of scale parameter points




%compute misspecification bias empirically
rho=.2;
SIGMA=[1 rho rho;rho 1 rho;rho rho 1];

X=mvnrnd([0 0 0],SIGMA,1e7);
x1=(exp(X(:,1))-exp(.5))/2.16; x2=X(:,2);
X2=[ones(1e7,1) x1 X(:,2:3)];
u=x1.*x2;
bias=X2\u;

clear X x1 x2 X2 u

%conduct simulations
results=NaN(K,8,S,2);



for n=1:3

N=2^(n-1)*100; %sample size    
    
%make wild bootstrap weights
%obeying E[W]=0, E[W^2]=1, E[W^3]=0
W=gaminv(rand(N,R),4,1/2)-2; %gamma weights
Wrad=2*(rand(N,R)<.5)-1; %rad weights
pos=rand(N,R)<.7236068;
Wmammen=-.61803399*pos + (1-pos)*(1+.61803399); %Mammen weights

%generate the data
%eta=exp(randn(N,R))-exp(.5); %recentered lognormal error
%eta=-log(rand(N,R))-1; %recentered exponential error
D=rand(N,R)<.1;

eta=((2*randn(N,R)+1).*D+(1-D).*randn(N,R)-.1);




x1=NaN(N,R);
x2=NaN(N,R);
x3=NaN(N,R);


for r=1:R
  thisX=mvnrnd([0 0 0],SIGMA,N);
  x1(:,r)=(exp(thisX(:,1))-exp(.5))/2.16;
  x2(:,r)=thisX(:,2);
  x3(:,r)=thisX(:,3);
end
  



dfk=N/(N-4);
one=ones(N,1);


pairs=NaN(N,R,R2);
for i=1:R
    for j=1:R2
        pairs(:,i,j)=randsample(N,N,1); %random pairs draw
    end
end


for s=1:S %vary scale parameter on error

e=(1+(s-1)*x1).*eta;

for k=1:K %vary degree of misspecification
k
gam=0+.5*(k-2); %misspecification parameter
Y=x1+x2+x3+gam*x1.*x2 + e; %DGP
%initialize results registers
rejectwild=NaN(R,1);
rejectwildrad=NaN(R,1);
rejectwildmammen=NaN(R,1);
rejectanal=NaN(R,1);
rejectpairs=NaN(R,1);

for i=1:R
    X=[one,x1(:,i),x2(:,i),x3(:,i)];
    xx=X'*X;
    xxi=inv(xx);

    b0=1+bias(2)*gam;
    
    %Get full sample stats
    [beta,ci,resid]=regress(Y(:,i),X);
    bigsig=xxi*X'*diag(resid.^2)*X*xxi*dfk;
    sig=sqrt(bigsig(2,2));
    tstat=abs((beta(2)-b0)/sig); %full sample t-stat

    
    %Bootstrap
    wildt=NaN(R2,1);
    wildradt=NaN(R2,1);
    wildmamment=NaN(R2,1);
    pairst=NaN(R2,1);
    
    %Wild and pairs bootstraps
    parfor r=1:R2 
	%Gamma
        Ystar=X*beta+resid.*W(:,r);
        [wildbeta,ci,wildresid]=regress(Ystar,X);
        wildbigsig=xxi*X'*diag(wildresid.^2)*X*xxi*dfk;
        wildsig=sqrt(wildbigsig(2,2));
        wildt(r,1)=abs((wildbeta(2)-beta(2))/wildsig); %wild t-stat (centered around full sample)

	%Rad
        Ystar=X*beta+resid.*Wrad(:,r);
        [wildbeta,ci,wildresid]=regress(Ystar,X);
        wildbigsig=xxi*X'*diag(wildresid.^2)*X*xxi*dfk;
        wildsig=sqrt(wildbigsig(2,2));
        wildradt(r,1)=abs((wildbeta(2)-beta(2))/wildsig); %wild Rad t-stat (centered around full sample)

	%Mammen
        Ystar=X*beta+resid.*Wmammen(:,r);
        [wildbeta,ci,wildresid]=regress(Ystar,X);
        wildbigsig=xxi*X'*diag(wildresid.^2)*X*xxi*dfk;
        wildsig=sqrt(wildbigsig(2,2));
        wildmamment(r,1)=abs((wildbeta(2)-beta(2))/wildsig); %wild Mammen t-stat (centered around full sample)
          
	%Pairs
        Ypairs=Y(pairs(:,i,r),i);
        Xpairs=X(pairs(:,i,r),:);
        xxipairs=inv(Xpairs'*Xpairs);
        [pairsbeta,ci,pairsresid]=regress(Ypairs,Xpairs);
        pairsbigsig=xxipairs*Xpairs'*diag(pairsresid.^2)*Xpairs*xxipairs*dfk;
        pairssig=sqrt(pairsbigsig(2,2));
        pairst(r,1)=abs((pairsbeta(2)-beta(2))/pairssig); %pairs t-stat (centered around full sample)
    end
    
    critT=quantile(wildt,.95);
    critTrad=quantile(wildradt,.95);
    critTmammen=quantile(wildmamment,.95);
    critTpairs=quantile(pairst,.95);
    
    rejectanal(i)=tstat>1.96;
    rejectwild(i)=tstat>critT;    
    rejectwildrad(i)=tstat>critTrad;
    rejectwildmammen(i)=tstat>critTmammen;
    rejectpairs(i)=tstat>critTpairs;
  

end

results(k,:,s,n)=[N,s,gam,mean(rejectanal),mean(rejectwild),mean(rejectwildrad),mean(rejectwildmammen),mean(rejectpairs)];
end

end

end


results


save results_twosided_web results

